#ifndef METRO_PLAN_H_INCLUDED
#define METRO_PLAN_H_INCLUDED

#include "metro/graph.hpp"
#include "metro/stationdata.h"

namespace metro
{

/**
 * \class Plan
 * 
 * cette classe repressente le plan du métro, avec ces lignes, ces stations.
 */
class Plan
{
	/// Graph
	typedef Graph<id_station>    graph_t;
	
	/// Solution
	typedef graph_t::solution_t  solution_t;
	
	/// Une ligne de métro est une liste de nom de station 
	typedef std::list<std::string>                   ligne_t;

	/// Liste de l'ensemble des lignes de métro
	typedef std::map<std::string, ligne_t>           plan_t;
	
	/// Ensemble des correspondance
	typedef std::multimap<std::string, std::string>  correspondances_t;
	
	/// Container d'étapes
	typedef std::vector<std::string>                 etapes_t;
	
	/// Permet de retrouver le nom réel du fichier pour une station nettoyée
	typedef std::map<std::pair<std::string, std::string>, std::string> file_names_t;

	plan_t            m_plan;
	correspondances_t m_correspondances;
	size_t            m_delaiStation;
	size_t            m_delaiCorrespondance;
	bool              m_bOptimise;
	etapes_t          m_etapes;
	std::list<int>    m_etapesFinales;
	std::set<int>     m_etapesId;
	file_names_t      m_fileNames;

public:
	Plan(size_t delaiStation, size_t delaiCorrespondance, bool bOptimise = false);
	
	~Plan() {
	}
	
	/// Charge le fichier des lignes
	bool Load(const std::string& filename);
	
	/// Assigne les etapes
	bool SetEtapes(const etapes_t& etapes);
	
	/// Calcul le trajet le plus court
	void Trajet(const std::string& from, const std::string& to);

	/// Verifie si la station existe sur le plan
	bool StationExists(const std::string& station) const;
	
protected:
	/// Ajoute une station à une ligne
	void Add(const std::string& ligne, const std::string& station);
	
	/// Crée le plan pour une étape
	void Create(graph_t& g, int& nEtape, const etapes_t& etapes);
	
	/// Crée une correspondance
	void CreateCorrespondance(graph_t& g, const StationData& stationData, int temps, int nEtape);
	
	/// La station est elle une etape
	bool IsStationEtape(const station::Data& stationData) const;
	
	/// Recupere le nom de la station tel qu'il est dans le fichier
	const std::string& GetFileStationName(const std::string& ligne, const std::string& station) const;
	
	/// Nettoie le nom de la station
	static std::string Clean(const std::string& station);
	
	/// Format l'heure
	static std::string FormatTemps(size_t temps);
};

} // namespace metro

#endif // METRO_PLAN_H_INCLUDED
