#include "metro/plan.h"
#include <iostream>

#include <boost/program_options.hpp>
namespace po = boost::program_options;

/**
 * Fonction appellée au debut du programme
 * 
 * \param argc  nombre d'arguments de la ligne de commande
 * \param argv  tableau contenant les arguments
 * 
 * \return code d'erreur ou SUCCES selon le fonctionnement du programme
 */
int main(int argc, char** argv)
{
	po::options_description desc("Options disponibles");
	desc.add_options()
		("help,h", "Affiche l'aide")
		("copyright,v", "Affiche le copyright")
		("file,f", po::value<std::string>(), "Fichier decrivant les lignes du metro")
		("delai-station,d", po::value<size_t>(), "Delai entre 2 stations en secondes")
		("delai-correspondance,c", po::value<size_t>(), "Delai entre 2 correspondances en secondes")
		("depart,x", po::value<std::string>(), "Nom de la station de depart")
		("arrivee,y", po::value<std::string>(), "Nom de la station d'arrivee")
		("intermediaires,i", po::value< std::vector<std::string> >(), "Nom des stations intermediaires")
		("optimise,o", "Optimise le trajet (ne repect pas forcement l'ordre des etapes)")
        ;
	
	po::variables_map vm;
	try {
		po::store(po::parse_command_line(argc, argv, desc), vm);
		po::notify(vm);    
	}
	catch (std::exception& e) {
		std::cerr << "Ligne de commande invalide :" << std::endl;
		std::cerr << e.what() << std::endl;
		return EXIT_FAILURE;
	}
	
	if (vm.count("help")) {
		std::cout << "Defi C++ Developpez.com : Balade dans le metro parisien." << std::endl;
		std::cout << desc << std::endl;
		return EXIT_SUCCESS;
	}
	
	if (vm.count("copyright")) {
		std::cout << "Defi C++ Developpez.com : Balade dans le metro parisien." << std::endl;
		std::cout << "Copyright Jeremie Fouche 2009 pour Developpez.com" << std::endl;
		std::cout << "Aucun droit sur ce logiciel"   << std::endl;
		return EXIT_SUCCESS;
	}
	
	if (!vm.count("delai-station")) {
		std::cerr << "Vous devez entrer le delai entre chaque stations" << std::endl;
		std::cerr << "Tapez metro --help pour l'aide" << std::endl;
		return EXIT_FAILURE;
	}
	size_t delayStation = vm["delai-station"].as<size_t>();
	
	if (!vm.count("delai-correspondance")) {
		std::cerr << "Vous devez entrer le delai entre chaque correspondance" << std::endl;
		std::cerr << "Tapez metro --help pour l'aide" << std::endl;
		return EXIT_FAILURE;
	}
	size_t delayCorrespondance = vm["delai-correspondance"].as<size_t>();
	
	if (!vm.count("file")) {
		std::cerr << "Vous devez entrer le fichier des stations" << std::endl;
		std::cerr << "Tapez metro --help pour l'aide" << std::endl;
		return EXIT_FAILURE;
	}
	std::string filename = vm["file"].as<std::string>();

	// On charge les stations
	metro::Plan plan(delayStation, delayCorrespondance, vm.count("optimise"));
	if (plan.Load(filename) == false) {
		std::cerr << "Erreur lors du chargement du fichier." << std::endl;
		return EXIT_FAILURE;
	}
	
	std::string depart;
	if (!vm.count("depart")) {
		std::cout << "Station de depart : " << std::endl << "> ";
		std::getline(std::cin, depart);
	}
	else {
		depart = vm["depart"].as<std::string>();
	}
	
	std::string arrivee;
	if (!vm.count("arrivee")) {
		std::cout << "Station d'arrivee : " << std::endl << "> ";
		std::getline(std::cin, arrivee);
	}
	else {
		arrivee = vm["arrivee"].as<std::string>();
	}

	if (vm.count("intermediaires")) {
		if (plan.SetEtapes(vm["intermediaires"].as< std::vector<std::string> >()) == false) {
			return EXIT_FAILURE;
		}
	}
	plan.Trajet(depart, arrivee);
	return EXIT_SUCCESS;
}
